'use strict';

var g = require('@antv/g');
var tslib = require('tslib');
var util = require('@antv/util');
var glMatrix = require('gl-matrix');
var React = require('react');
var ReactReconciler = require('react-reconciler');
var scheduler = require('scheduler');

var DEFAULT_ARROW_SIZE = 16;
var Arrow = /** @class */ (function (_super) {
    tslib.__extends(Arrow, _super);
    function Arrow(props) {
        var _this = _super.call(this, props) || this;
        _this.startRef = React.createRef();
        _this.endRef = React.createRef();
        _this.bodyRef = React.createRef();
        return _this;
    }
    Arrow.prototype.getArrowHead = function (head, isStart) {
        if (util.isBoolean(head)) {
            return this.getDefaultArrowHead();
        }
        else {
            return isStart ? this.props.startHead : this.props.endHead;
        }
    };
    Arrow.prototype.setHeadTransform = function () {
        var _a = this.props, startHead = _a.startHead, endHead = _a.endHead;
        if (startHead)
            this.transformArrowHead(this.startRef.current, true);
        if (endHead)
            this.transformArrowHead(this.endRef.current, false);
    };
    Arrow.prototype.componentDidMount = function () {
        this.setHeadTransform();
    };
    Arrow.prototype.componentDidUpdate = function () {
        this.setHeadTransform();
    };
    Arrow.prototype.render = function () {
        var _a = this.props, startHead = _a.startHead, endHead = _a.endHead, others = tslib.__rest(_a, ["startHead", "endHead"]);
        return (React.createElement(Group, null,
            React.createElement(Path, tslib.__assign({}, others, { ref: this.bodyRef })),
            startHead && React.createElement(Group, { ref: this.startRef }, this.getArrowHead(startHead, true)),
            endHead && React.createElement(Group, { ref: this.endRef }, this.getArrowHead(endHead, false))));
    };
    Arrow.prototype.getCenter = function () {
        var points = this.bodyRef.current.getPoint(0.5);
        return points;
    };
    // transform arrow head to match line tangent
    Arrow.prototype.transformArrowHead = function (head, isStart) {
        var _a = tslib.__read(this.getTangent(this.bodyRef.current, isStart), 2), p1 = _a[0], p2 = _a[1];
        var _b = tslib.__read(p1, 2), x1 = _b[0], y1 = _b[1];
        var _c = tslib.__read(p2, 2), x2 = _c[0], y2 = _c[1];
        var x = x1 - x2;
        var y = y1 - y2;
        var rad = Math.atan2(y, x) + Math.PI;
        var position = glMatrix.vec3.fromValues(x2, y2, 0);
        head.setLocalPosition(position);
        head.setLocalEulerAngles((rad * 180) / Math.PI);
    };
    Arrow.prototype.getTangent = function (path, isStart) {
        return isStart ? path.getStartTangent() : path.getEndTangent();
    };
    Arrow.prototype.getDefaultArrowHead = function () {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        var _a = this.props; _a.startHead; _a.endHead; var others = tslib.__rest(_a, ["startHead", "endHead"]);
        var sin = Math.sin, cos = Math.cos, PI = Math.PI;
        return (React.createElement(Path, tslib.__assign({}, others, { lineDash: undefined, fill: this.props.stroke, path: "M-".concat(DEFAULT_ARROW_SIZE * cos(PI / 6), ",").concat(DEFAULT_ARROW_SIZE * sin(PI / 6), " L0,0 L-").concat(DEFAULT_ARROW_SIZE * cos(PI / 6), ",-").concat(DEFAULT_ARROW_SIZE * sin(PI / 6), " Z") })));
    };
    return Arrow;
}(React.Component));

function ElementOf(type) {
    return type;
}
var Circle = ElementOf(g.Shape.CIRCLE);
var Ellipse = ElementOf(g.Shape.ELLIPSE);
var Group = ElementOf(g.Shape.GROUP);
var HTML = ElementOf(g.Shape.HTML);
var Image = ElementOf(g.Shape.IMAGE);
var Line = ElementOf(g.Shape.LINE);
var Path = ElementOf(g.Shape.PATH);
var Polygon = ElementOf(g.Shape.POLYGON);
var Polyline = ElementOf(g.Shape.POLYLINE);
var Rect = ElementOf(g.Shape.RECT);
var Text = ElementOf(g.Shape.TEXT);

var zIndexWarningShowed = false;
var Z_INDEX_WARNING = "ReactG: You are using \"zIndex\" attribute for an element.\nreact-g may get confused with ordering. Just define correct order of elements in your render function of a component.\n";
var isEvent = function (key) { return key.slice(0, 2) === 'on'; };
var getEventName = function (key) { return key.substring(2).toLowerCase(); };
var bindShapeEvent = function (newProps, instance) {
    Object.keys(newProps).forEach(function (propKey) {
        if (isEvent(propKey)) {
            instance.addEventListener(getEventName(propKey), newProps[propKey]);
        }
    });
};
var updateProps = function (instance, newProps, oldProps) {
    if (oldProps === void 0) { oldProps = {}; }
    // don't use zIndex
    if (!zIndexWarningShowed && 'zIndex' in newProps) {
        console.warn(Z_INDEX_WARNING);
        zIndexWarningShowed = true;
    }
    Object.keys(oldProps).forEach(function (key) {
        var propChanged = oldProps[key] !== newProps[key];
        if (propChanged) {
            if (isEvent(key)) {
                instance.removeEventListener(getEventName(key));
            }
            else {
                instance.removeAttribute(key);
            }
        }
    });
    Object.keys(newProps).forEach(function (key) {
        var propChanged = oldProps[key] !== newProps[key];
        if (propChanged) {
            if (isEvent(key)) {
                if (typeof newProps[key] === 'function') {
                    instance.addEventListener(getEventName(key), newProps[key]);
                }
            }
            else {
                instance.setAttribute(key, newProps[key]);
            }
        }
    });
};

var reconcilor = ReactReconciler({
    getPublicInstance: function (instance) {
        return instance;
    },
    getRootHostContext: function (rootContainerInstance) { },
    getChildHostContext: function (parentHostContext, type, rootContainerInstance) { },
    prepareForCommit: function (containerInfo) {
        return null;
    },
    resetAfterCommit: function (containerInfo) { },
    preparePortalMount: function (containerInfo) { },
    createInstance: function (type, props, rootContainerInstance, hostContext, internalInstanceHandle) {
        var instance = (rootContainerInstance.document ||
            rootContainerInstance.ownerDocument).createElement(type, {
            style: props,
        });
        // @ts-ignore
        bindShapeEvent(props, instance);
        // log('createInstance ', type, instance);
        // @ts-ignore
        return instance;
    },
    appendInitialChild: function (parentInstance, child) {
        // log('appendInitialChild', parentInstance, child);
        parentInstance.appendChild(child);
    },
    finalizeInitialChildren: function (parentInstance, type, props, rootContainerInstance, hostContext) {
        return false;
    },
    prepareUpdate: function (instance, type, oldProps, newProps, rootContainerInstance, hostContext) {
        // return hasUpdate(newProps, oldProps);
        return true;
    },
    shouldSetTextContent: function (type, props) {
        return false;
    },
    // shouldDeprioritizeSubtree(type: Type, props: Props): boolean {
    //   return false;
    // },
    createTextInstance: function (text, rootContainerInstance, hostContext, internalInstanceHandle) { },
    // scheduleDeferredCallback(callback: () => any, options?: { timeout: number }): any {},
    // scheduleDeferredCallback,
    // cancelDeferredCallback(callbackID: any): void {},
    scheduleTimeout: function (handler, timeout) { },
    cancelTimeout: function (handle) { },
    noTimeout: undefined,
    now: scheduler.unstable_now,
    // Temporary workaround for scenario where multiple renderers concurrently
    // render using the same context objects. E.g. React DOM and React ART on the
    // same page. DOM is the primary renderer {}, ART is the secondary renderer.
    isPrimaryRenderer: false,
    supportsMutation: true,
    supportsPersistence: false,
    supportsHydration: false,
    // -------------------
    //      Mutation
    //     (optional)
    // -------------------
    appendChild: function (parentInstance, child) {
        // log('appendChild');
        parentInstance.appendChild(child);
    },
    appendChildToContainer: function (container, child) {
        // log('appendChildToContainer', container, child);
        container.appendChild(child);
    },
    commitTextUpdate: function (textInstance, oldText, newText) { },
    commitMount: function (instance, type, newProps, internalInstanceHandle) { },
    commitUpdate: function (instance, updatePayload, type, oldProps, newProps, internalInstanceHandle) {
        // log('commitUpdate', instance, newProps);
        updateProps(instance, newProps, oldProps);
    },
    insertBefore: function (parentInstance, child, beforeChild) {
        parentInstance.insertBefore(child, beforeChild);
    },
    insertInContainerBefore: function (container, child, beforeChild) {
        container.insertBefore(child, beforeChild);
    },
    removeChild: function (parentInstance, child) {
        // log('removeChild', parentInstance, child);
        parentInstance.removeChild(child);
    },
    removeChildFromContainer: function (container, child) {
        container.removeChild(child);
    },
    resetTextContent: function (instance) { },
    /**
     * This method should make the `instance` invisible without removing it from the tree. For example, it can apply visual styling to hide it. It is used by Suspense to hide the tree while the fallback is visible.
     */
    // tslint:enable:max-line-length
    hideInstance: function (instance) { },
    /**
     * Same as `hideInstance`, but for nodes created by `createTextInstance`.
     */
    hideTextInstance: function (textInstance) { },
    /**
     * This method should make the `instance` visible, undoing what `hideInstance` did.
     */
    // @ts-ignore
    unhideInstance: function (instance, props) { },
    /**
     * Same as `unhideInstance`, but for nodes created by `createTextInstance`.
     */
    // @ts-ignore
    unhideTextInstance: function (textInstance, text) { },
    /**
     * This method should mutate the `container` root node and remove all children from it.
     */
    clearContainer: function (container) {
        container.removeChildren();
    },
    // -------------------
    //     Persistence
    //     (optional)
    // -------------------
    cloneInstance: function (instance, updatePayload, type, oldProps, newProps, internalInstanceHandle, keepChildren, recyclableInstance) {
        return instance;
    },
    createContainerChildSet: function (container) { },
    appendChildToContainerChildSet: function (childSet, child) { },
    finalizeContainerChildren: function (container, newChildren) { },
    replaceContainerChildren: function (container, newChildren) { },
    // -------------------
    //     Hydration
    //     (optional)
    // -------------------
    canHydrateInstance: function (instance, type, props) {
        return instance;
    },
    canHydrateTextInstance: function (instance, text) {
        return null;
    },
    getNextHydratableSibling: function (instance) { },
    getFirstHydratableChild: function (parentInstance) { },
    hydrateInstance: function (instance, type, props, rootContainerInstance, hostContext, internalInstanceHandle) { },
    hydrateTextInstance: function (textInstance, text, internalInstanceHandle) {
        return false;
    },
    didNotMatchHydratedContainerTextInstance: function (parentContainer, textInstance, text) { },
    didNotMatchHydratedTextInstance: function (parentType, parentProps, parentInstance, textInstance, text) { },
    didNotHydrateContainerInstance: function (parentContainer, instance) { },
    didNotHydrateInstance: function (parentType, parentProps, parentInstance, instance) { },
    didNotFindHydratableContainerInstance: function (parentContainer, type, props) { },
    didNotFindHydratableContainerTextInstance: function (parentContainer, text) { },
    didNotFindHydratableInstance: function (parentType, parentProps, parentInstance, type, props) { },
    didNotFindHydratableTextInstance: function (parentType, parentProps, parentInstance, text) { },
});
reconcilor.injectIntoDevTools({
    // findFiberByHostInstance: () => {},
    // @ts-ignore
    bundleType: process.env.NODE_ENV !== 'production' ? 1 : 0,
    version: React.version,
    rendererPackageName: 'react-g',
    rendererConfig: {
        getInspectorDataForViewTag: function (tag) {
            // console.log(tag);
        },
    },
});
/**
 * render react-g component to target g element
 * 将react-g组件渲染到任意的g实例（Canvas/Group/Shape）中
 * @param component react-g component
 * @param target g element, Canvas/Group/Shape instance
 * @param callback callback after render finished
 * @returns void
 */
var render = function (component, target, callback) {
    // @ts-ignore
    var container = reconcilor.createContainer(target, 1, false, null);
    reconcilor.updateContainer(component, container, null, callback);
};

function assertRef(forwardedRef) {
    if (typeof forwardedRef === 'function') {
        throw new Error('Callback ref not support!');
    }
}

var Canvas = React.forwardRef(function (_a, ref) {
    var children = _a.children, classname = _a.classname, role = _a.role, style = _a.style, tabIndex = _a.tabIndex, title = _a.title, renderer = _a.renderer, width = _a.width, height = _a.height, capture = _a.capture, cursor = _a.cursor;
    assertRef(ref);
    var container = React.useRef();
    var divRef = React.useRef(null);
    var innerCanvasRef = React.useRef(null);
    var canvasRef = ref || innerCanvasRef;
    React.useLayoutEffect(function () {
        var canvas = new g.Canvas({
            renderer: renderer,
            width: width,
            height: height,
            capture: capture,
            cursor: cursor,
            container: divRef.current,
        });
        canvasRef.current = canvas;
        // @ts-ignore
        container.current = reconcilor.createContainer(canvas, 1, false, null);
        return function () {
            // @ts-ignore
            reconcilor.updateContainer(null, container.current, null);
        };
    }, []);
    React.useLayoutEffect(function () {
        if (container.current) {
            // @ts-ignore
            reconcilor.updateContainer(children, container.current, null);
        }
    }, [children]);
    React.useLayoutEffect(function () {
        var _a;
        (_a = canvasRef.current) === null || _a === void 0 ? void 0 : _a.setRenderer(renderer);
    }, [canvasRef, renderer]);
    React.useLayoutEffect(function () {
        var _a;
        (_a = canvasRef.current) === null || _a === void 0 ? void 0 : _a.setCursor(cursor);
    }, [canvasRef, cursor]);
    React.useLayoutEffect(function () {
        canvasRef.current.resize(width, height);
    }, [width, height, canvasRef]);
    return (React.createElement("div", { ref: divRef, className: classname, role: role, style: style, tabIndex: tabIndex, title: title }));
});

exports.Arrow = Arrow;
exports.Canvas = Canvas;
exports.Circle = Circle;
exports.ElementOf = ElementOf;
exports.Ellipse = Ellipse;
exports.Group = Group;
exports.HTML = HTML;
exports.Image = Image;
exports.Line = Line;
exports.Path = Path;
exports.Polygon = Polygon;
exports.Polyline = Polyline;
exports.Rect = Rect;
exports.Text = Text;
exports.reconcilor = reconcilor;
exports.render = render;
//# sourceMappingURL=index.js.map
